@@ -84,7 +84,8 @@
 #'
 #' @param x input matrix, of dimension nobs x nvars; each row is an observation
 #' vector. Can be in sparse matrix format (inherit from class
-#' \code{"sparseMatrix"} as in package \code{Matrix})
+#' \code{"sparseMatrix"} as in package \code{Matrix}).
+#' Requirement: \code{nvars >1}; in other words, \code{x} should have 2 or more columns.
 #' @param y response variable. Quantitative for \code{family="gaussian"}, or
 #' \code{family="poisson"} (non-negative counts). For \code{family="binomial"}
 #' should be either a factor with two levels, or a two-column matrix of counts
@@ -234,20 +235,21 @@
 #' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
 #' \emph{Regularization Paths for Generalized Linear Models via Coordinate
 #' Descent (2010), Journal of Statistical Software, Vol. 33(1), 1-22},
-#' \url{https://web.stanford.edu/~hastie/Papers/glmnet.pdf}.\cr
+#' \doi{10.18637/jss.v033.i01}.\cr
 #' Simon, N., Friedman, J., Hastie, T. and Tibshirani, R. (2011)
 #' \emph{Regularization Paths for Cox's Proportional
 #' Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
-#' 39(5), 1-13}, \url{https://www.jstatsoft.org/v39/i05/}.\cr Tibshirani,
-#' Robert, Bien, J., Friedman, J., Hastie, T.,Simon, N.,Taylor, J. and
+#' 39(5), 1-13},
+#' \doi{10.18637/jss.v039.i05}.\cr
+#' Tibshirani,Robert, Bien, J., Friedman, J., Hastie, T.,Simon, N.,Taylor, J. and
 #' Tibshirani, Ryan. (2012) \emph{Strong Rules for Discarding Predictors in
 #' Lasso-type Problems, JRSSB, Vol. 74(2), 245-266},
-#' \url{https://statweb.stanford.edu/~tibs/ftp/strong.pdf}.\cr
-#' Hastie, T., Tibshirani, Robert and Tibshirani, Ryan. \emph{Extended
-#' Comparisons of Best Subset Selection, Forward Stepwise Selection, and the
-#' Lasso (2017), Stanford Statistics Technical Report},
+#' \url{https://arxiv.org/abs/1011.2234}.\cr
+#' Hastie, T., Tibshirani, Robert and Tibshirani, Ryan (2020) \emph{Best Subset,
+#' Forward Stepwise or Lasso? Analysis and Recommendations Based on Extensive Comparisons,
+#' Statist. Sc. Vol. 35(4), 579-592},
 #' \url{https://arxiv.org/abs/1707.08692}.\cr
-#' Glmnet webpage with four vignettes, \url{https://glmnet.stanford.edu}.
+#' Glmnet webpage with four vignettes: \url{https://glmnet.stanford.edu}.
 #' @keywords models regression
 #' @examples
 #'
@@ -352,6 +354,8 @@ glmnet=function(x,y,family=c("gaussian","binomial","poisson","multinomial","cox"
     if(is.null(np)|(np[2]<=1))stop("x should be a matrix with 2 or more columns")
     nobs=as.integer(np[1])
     nvars=as.integer(np[2])
+    ##check for NAs
+    if(any(is.na(x)))stop("x has missing values; consider using makeX() to impute them")
     if(is.null(weights))weights=rep(1,nobs)
     else if(length(weights)!=nobs)stop(paste("number of elements in weights (",length(weights),") not equal to the number of rows of x (",nobs,")",sep=""))
     if(is.function(exclude))exclude <- check.exclude(exclude(x=x,y=y,weights=weights),nvars)
@@ -463,19 +467,34 @@ glmnet=function(x,y,family=c("gaussian","binomial","poisson","multinomial","cox"
       if(inherits(x,"sparseMatrix")){##Sparse case
         is.sparse=TRUE
         x=as(x,"CsparseMatrix")
-        x=as(x,"dgCMatrix")
+        x=as(x,"dMatrix")
+#        x=as(x,"generalMatrix")
         ix=as.integer(x@p+1)
         jx=as.integer(x@i+1)
-        xd=x@x
+
+        # TODO: changed everything except cox to C++ implementation.
+        # C++ version takes xd as the dgCMatrix itself.
+        # Fortran requires xd to be the compressed data vector.
+        if (family != "cox") {
+            xd <- x
+        } else {
+            xd <- x@x
+        }
+
       } else if (!inherits(x, "matrix")) {
         xd <- data.matrix(x)
       } else {
         xd <- x
       }
-      storage.mode(xd) <- "double"
+      # TODO: only coerce if xd is not sparse
+      if(!inherits(xd,"sparseMatrix")) {
+        storage.mode(xd) <- "double"
+      }
       if (trace.it) {
         if (relax) cat("Training Fit\n")
         pb  <- createPB(min = 0, max = nlam, initial = 0, style = 3)
+      } else {
+        pb <- NULL # dummy initialize (won't be used, but still need to pass)
       }
       kopt=switch(match.arg(type.logistic),
                   "Newton"=0,#This means to use the exact Hessian
@@ -488,12 +507,12 @@ glmnet=function(x,y,family=c("gaussian","binomial","poisson","multinomial","cox"
       kopt=as.integer(kopt)
 
       fit=switch(family,
-                 "gaussian"=elnet(xd,is.sparse,ix,jx,y,weights,offset,type.gaussian,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit),
-                 "poisson"=fishnet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit),
-                 "binomial"=lognet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family),
-                 "multinomial"=lognet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family),
+                 "gaussian"=elnet(xd,is.sparse,y,weights,offset,type.gaussian,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,pb),
+                 "poisson"=fishnet(xd,is.sparse,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,pb),
+                 "binomial"=lognet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family,pb),
+                 "multinomial"=lognet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family,pb),
                  "cox"=coxnet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,vnames,maxit),
-                 "mgaussian"=mrelnet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,jsd,intr,vnames,maxit)
+                 "mgaussian"=mrelnet(xd,is.sparse,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,jsd,intr,vnames,maxit,pb)
                  )
       if (trace.it) {
         utils::setTxtProgressBar(pb, nlam)
